from Acquisition import aq_inner
from DateTime import DateTime
from Products.CMFCore.utils import getToolByName
from Products.Five.browser import pagetemplatefile
from Products.statusmessages.interfaces import IStatusMessage
from slc.xliff import XliffMessageFactory as _
from slc.xliff.interfaces import IExportParams
from slc.xliff.interfaces import IImportParams
from slc.xliff.interfaces import IXLIFFExporter
from slc.xliff.interfaces import IXLIFFImporter
from z3c.form import button
from z3c.form import field
from z3c.form import form
from zope.component import getUtility
from Products.Five.browser import BrowserView


class ExportXliffForm(form.Form):
    """ Form for exporting xliff
    """
    fields = field.Fields(IExportParams)
    ignoreContext = True
    label = u'Export Xliff'
    form_name = _(u"Export Xliff")

    def __call__(self):
        self.request.set('disable_border', 'on')
        return super(ExportXliffForm, self).__call__()

    def setUpWidgets(self, ignore_request=False):
        self.adapters = {}
        data = {}
        # make the recursive checked if context is folderish
        if self.context.isPrincipiaFolderish:
            data = {'recursive': True}
        self.widgets = setUpWidgets(
            self.form_fields, self.prefix, self.context, self.request,
            form=self, data=data, adapters=self.adapters, ignore_request=ignore_request)

    @button.buttonAndHandler(u'Export')
    def action_export(self, action):
        context = aq_inner(self.context)
        data, errors = self.extractData()
        recursive = int(bool(data.get('recursive')))
        single_file = int(bool(data.get('single_file')))
        zip = int(bool(data.get('zip')))
        html_compatibility = int(bool(data.get('html_compatibility')))

        return self.request.response.redirect(
            self.context.absolute_url() + '/@@export_xliff_contents?recursive={}&single_file={}&zip={}&html_compatibility={}'.format(
                recursive, single_file, zip, html_compatibility)
        )


class ImportXliffForm(form.Form):
    """ Form for importing xliff
    """
    fields = field.Fields(IImportParams)
    ignoreContext = True

    label = _(u"Import Xliff")

    @button.buttonAndHandler(_(u'Import'))
    def handleApply(self, action):
        data, errors = self.extractData()
        context = aq_inner(self.context)
        xliff_file = data['xliff_file']
        xliffimporter = getUtility(IXLIFFImporter)
        errors = xliffimporter.upload(
            xliff_file, html_compatibility=False, request=context.REQUEST)
        if errors != []:
            error = ["%s: %s" % x for x in errors]
            confirm = _(u"Error while importing Xliff.\n " + "\n".join(error))
            IStatusMessage(self.request).addStatusMessage(confirm, type='warn')
        else:
            confirm = _(u"Xliff import successful.")
            IStatusMessage(self.request).addStatusMessage(confirm, type='info')

        self.request.response.redirect(context.absolute_url() + '/@@xliffimport')

        return ''


class XliffExportView(BrowserView):
    def __call__(self):
        data = self.request.form
        recursive = bool(int(data.get('recursive')))
        single_file = bool(int(data.get('single_file')))
        zip = bool(int(data.get('zip')))
        html_compatibility = bool(int(data.get('html_compatibility')))

        context = aq_inner(self.context)
        if self.context.isPrincipiaFolderish:
            container = context
        else:
            container = context.aq_parent

        if recursive is True:
            xliffexporter = IXLIFFExporter(container)
        else:
            xliffexporter = IXLIFFExporter(context)

        xliffexporter.recursive = recursive
        xliffexporter.single_file = single_file
        xliffexporter.html_compatibility = html_compatibility
        xliffexporter.zip = zip
        xliffexporter.source_language = "en"

        data = xliffexporter.export()

        if zip is True:
            self.request.response.setHeader('Content-type', 'application/zip')
            self.request.response.setHeader('Content-Disposition',
                                            'attachment; filename=xliff_export_{0}.zip'.format(DateTime().strftime('%Y-%m-%d')))
        elif html_compatibility and single_file:
            self.request.response.setHeader('Content-type', 'text/html')
            self.request.response.setHeader('Content-Disposition',
                                            'attachment; filename={0}_xliff.html'.format(context.getId()))
        elif single_file:
            self.request.response.setHeader('Content-type', 'text/xml')
            self.request.response.setHeader('Content-Disposition',
                                            'attachment; filename={0}.xliff'.format(context.getId()))
        else:
            pass    # Should not happen
        self.request.response.write(data)
        return self.request.response



# -*- coding: utf-8 -*-
from plone.namedfile.field import NamedFile
from zope.interface import Interface
from zope import schema
from slc.xliff import XliffMessageFactory as _
from zope.publisher.interfaces.browser import IDefaultBrowserLayer


class ISlcXliff(IDefaultBrowserLayer):
    """Marker interface that defines a browser layer."""

class IExportParams(Interface):
    """ Management form to export xliff """

    recursive = schema.Bool(
        title=u'Recursive subtree export?',
        description=_(
            u'Use this if you want to export everything below that '
            u'folder. If you use the recursive option, you can restrict your '
            u'export by specifying additional constraints below.'),
        required=False,
        default=False,
    )

    single_file = schema.Bool(
        title=u'Generate a single file?',
        description=_(u'Use this if you want to export everything in one file.'),
        required=False,
        default=True,
    )

    zip = schema.Bool(
        title=u'Download as zip?',
        description=_(u'Use this if you want to download the export as zip file.'),
        required=False,
        default=True,
    )

    html_compatibility = schema.Bool(
        title=u'HTML compatibility mode',
        description=_(u'Use this if you want an HTML-ish export that can be read by HTML editors.'),
        required=False,
        default=False,
    )


class IImportParams(Interface):
    """ Management form to import xliff """

    html_compatibility = schema.Bool(
        title=u'HTML compatibility mode',
        description=_(
            u'Select this option if your files are messed up with HTML code '
            u'in between due to incorrect XLIFF file handling by the '
            u'translators.'),
        required=False,
        default=False,
    )

    xliff_file = NamedFile(title=u'Xliff file (plain or zip)', required=True)


class IXLIFFExporter(Interface):
    """ An Adapter that can create an xliff representation from objects
    """

    def export():
        """ generate the xliff representation of this object
        """


class IXLIFFImporter(Interface):
    """ An Utility that can set xliff translations onto objects
    """

    def upload(data, html_compatibility=False):
        """ write one or more xliff documents from a file or zip onto objects
        """


class IXLIFF(Interface):
    """ render an objects xliff representation
    """

    def render():
        """ render an objects xliff representation
        """

    def set_translation(attrs):
        """ set translated attributes
        """


class IAttributeExtractor(Interface):
    """ Extracts attributes to translate from a given object """



from plone.app.contenttypes.testing import PLONE_APP_CONTENTTYPES_FIXTURE
from plone.app.multilingual.testing import MultipleLanguagesLayer
from plone.app.multilingual.testing import PLONE_APP_MULTILINGUAL_PRESET_FIXTURE
from plone.app.testing import applyProfile
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import login
from plone.app.testing import PloneSandboxLayer
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.testing import z2
from Products.Five import fiveconfigure
from Products.CMFPlone.utils import getToolByName


class SlcXliffLayer(PloneSandboxLayer):
    defaultBases = (PLONE_APP_CONTENTTYPES_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        super(SlcXliffLayer, self).setUpZope(app, configurationContext)
        # z2.installProduct(app, "Products.ZCatalog")

        fiveconfigure.debug_mode = True
        import plone.app.multilingual

        self.loadZCML("configure.zcml", package=plone.app.multilingual)

        import slc.xliff

        self.loadZCML("configure.zcml", package=slc.xliff)
        self.loadZCML("testing.zcml", package=slc.xliff)

        fiveconfigure.debug_mode = False
        z2.installProduct(app, "slc.xliff")

    def setUpPloneSite(self, portal):
        portal.acl_users.userFolderAddUser(
            SITE_OWNER_NAME, SITE_OWNER_PASSWORD, ["Manager"], []
        )
        login(portal, SITE_OWNER_NAME)
        setRoles(portal, TEST_USER_ID, ["Manager"])

        language_tool = getToolByName(portal, "portal_languages")
        language_tool.addSupportedLanguage("en")
        language_tool.addSupportedLanguage("de")
        language_tool.addSupportedLanguage("es")
        language_tool.addSupportedLanguage("fi")

        applyProfile(portal, "plone.app.multilingual:default")
        applyProfile(portal, "slc.xliff:default")
        applyProfile(portal, "slc.xliff:testing")

        # Commit so that the test browser sees these objects
        # portal.portal_catalog.clearFindAndRebuild()
        # import transaction

        # transaction.commit()

    # def tearDownZope(self, app):
    #     z2.uninstallProduct(app, "Products.ZCatalog")


SLC_XLIFF_FIXTURE = SlcXliffLayer()
INTEGRATION_TESTING = IntegrationTesting(
    bases=(SLC_XLIFF_FIXTURE,), name="SlcXliff:Integration"
)
FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(SLC_XLIFF_FIXTURE,), name="SlcXliff:Functional"
)



from zope import schema
from zope.component import adapts
from zope.interface import alsoProvides

from plone.app.textfield import RichText
from plone.app.multilingual.dx.interfaces import ILanguageIndependentField
from plone.supermodel import model

from slc.xliff import XliffMessageFactory as _
from slc.xliff.adapters.dx import BaseDXAttributeExtractor


class ITestContent(model.Schema):

    text = RichText(
        title=_("Text"),
        required=False,
    )

    language_independent = schema.TextLine(
        title=_(u"Language Independent"),
        required=False,
    )

alsoProvides(ITestContent['language_independent'], ILanguageIndependentField)


class TestContentExtractor(BaseDXAttributeExtractor):
    adapts(ITestContent)
    attrs = ['title', 'description', 'text']



HTML_HEAD = """<html>
<meta http-equiv="Content-Type" content="text/html;charset=utf-8" />
<xliff version='1.2'
       xmlns='urn:oasis:names:tc:xliff:document:1.2'
       xmlns:tal="http://xml.zope.org/namespaces/tal"
       xmlns:metal="http://xml.zope.org/namespaces/metal">
{content}
</xliff>
</html>
"""

HTML_FILE_BODY = """<file original="{original}" oid="{oid}" source-language="{source_language}" target-language="{source_language}">
<body>
<group>
{attrs}
</group>
</body>
</file>
"""

HTML_ATTR_BODY = """    <trans-unit id="{id}">
        <source xml:lang='{source_language}'>
        <!-- {value} -->
        </source>
        <target xml:lang='{source_language}'>
        {value}
        </target>
    </trans-unit>
"""



# -*- coding: utf-8 -*-
from Acquisition import aq_inner
from bs4 import BeautifulSoup
from plone import api
from plone.app.multilingual.dx.interfaces import ILanguageIndependentField
from plone.app.multilingual.interfaces import ITranslatable
from plone.app.multilingual.interfaces import ITranslationManager
from plone.app.textfield.interfaces import IRichText
from plone.app.textfield.value import RichTextValue
from plone.behavior.interfaces import IBehaviorAssignable
from plone.dexterity.interfaces import IDexterityContent
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.utils import safe_unicode
from Products.statusmessages.interfaces import IStatusMessage
from six import BytesIO
from six import StringIO
from slc.xliff.interfaces import IAttributeExtractor
from slc.xliff.interfaces import IXLIFF
from slc.xliff.interfaces import IXLIFFExporter
from slc.xliff.interfaces import IXLIFFImporter
from slc.xliff.templates.html import HTML_FILE_BODY
from slc.xliff.templates.html import HTML_HEAD
from slc.xliff.templates.xliff import XLIFF_FILE_BODY
from slc.xliff.templates.xliff import XLIFF_HEAD
from zope.component import adapter
from zope.interface import implementer
from zope.schema import getFieldsInOrder
from zope.site.hooks import getSite

import logging
import six.moves.html_parser
import zipfile


logger = logging.getLogger('slc.xliff')
html_parser = six.moves.html_parser.HTMLParser()


def get_dx_schema(context):
    schema = dict(getFieldsInOrder(context.getTypeInfo().lookupSchema()))

    behavior_assignable = IBehaviorAssignable(context)
    if behavior_assignable:
        for behavior in behavior_assignable.enumerateBehaviors():
            for k, v in getFieldsInOrder(behavior.interface):
                schema[k] = v

    return schema


@implementer(IXLIFFImporter)
class XLIFFImporter(object):
    """ utility to reimport xliff translations and create/edit the respective
    objects """

    def upload(self, xliff_file, html_compatibility=False, request=None):
        """ write one or more xliff documents from a file or zip onto objects
        """
        self.request = request
        # data maybe a zip file or a plain file.
        # A zip may contain one or more files
        # each file may contain one or more file sections
        filelist = []
        filetype = ''
        errors = []
        self.total = 0
        self.new_translations = 0
        filename = xliff_file.filename
        data = xliff_file.data

        major, minor = xliff_file.contentType.split('/')
        if major == 'text':
            filetype = 'plain'
        elif major == 'application' and minor in ['zip', 'x-zip-compressed']:
            filetype = 'zip'
        else:
            errors.append(('File error', "File type unknown"))
            return errors

        if filetype == 'zip':
            Z = BytesIO()
            Z.write(data)
            zf = zipfile.ZipFile(Z, 'r')
            testzip = zf.testzip()
            if testzip:
                errors.append(('File error', "Zip file corrupt?"))
            nameList = zf.namelist()
            for i in nameList:
                content = zf.read(i)
                if len(content) == 0:
                    continue
                filelist.append((i, content))
            zf.close()
            Z.close()

        else:
            filelist.append((filename, data))

        for xliff in filelist:
            soup = BeautifulSoup(xliff[1])
            target_language = _guessLanguage(xliff[0])
            if target_language and self.request is not None:
                msg = u"Detected language from file name: {0}".format(
                    target_language)
                IStatusMessage(self.request).addStatusMessage(msg, type='info')
            file_sections = soup.findAll('file')
            if soup.findAll('file') == []:
                errors.append((
                    'Empty File?', '{0} contains no file sections.'.format(xliff[0])
                    ))

            for section in file_sections:
                if api.env.debug_mode():
                    self._setXLIFF(section, target_language=target_language)
                else:
                    try:
                        self._setXLIFF(
                            section, target_language=target_language)
                    except ValueError as ve:
                        errors.append(('Target Object', str(ve)))
                    except Exception as e:
                        errors.append(('General Exception', str(e)))
                        raise e

                # We don't do intermediate commits any more, too many ConflictErrors
                #transaction.commit()
            if self.request is not None:
                msg = u"Handled a total of {0} files, of which {1} were new translations".format(
                    self.total, self.new_translations)
                IStatusMessage(self.request).addStatusMessage(msg, type='info')
        return errors

    def _setXLIFF(self, data, target_language=''):
        """ Set the data on one object """
        site = getSite()
        portal_catalog = getToolByName(site, 'portal_catalog')

        if target_language == '':
            target_language = data['target-language']

        # nothing to do if there is no target language
        if not target_language:
            return

        try:
            oid = data['oid']
            results = portal_catalog(UID=oid)
            if len(results) != 1:
                #raise ValueError, "Uid catalog should return exactly one
                #result but returned %s." % len(results)
                raise KeyError("Invalid OID {0}".format(oid))
            source_ob = results[0].getObject()
        except KeyError:
            # old style xliff file. Using path
            #print "Using path to find target object"
            path = data['original']
            source_ob = site.restrictedTraverse(path, None)

        if source_ob is None:
            raise ValueError(
                "{0} not found, can not add translation.".format(data['original']))

        # If the source object is language-neutral, it must receive a language
        # prior to translation
        # XXXX What to do if the source_ob HAS a language, but it differs
        # from the source-language inside the data?
        if source_ob.Language() == '':
            # Get the source language from the data
            source_language = data.get('source-language')
            # No source language present in the section, use site default
            if not source_language:
                langtool = getToolByName(site, 'portal_languages')
                source_language = langtool.getPreferredLanguage()
            source_ob.setLanguage(source_language)

        tm = ITranslationManager(source_ob)
        if not tm.has_translation(target_language):
            tm.add_translation(target_language)
            self.new_translations += 1
        target_ob = tm.get_translation(target_language)

        values = {}

        for unit in data.findAll('trans-unit'):

            fieldname = unit['id']
            value = unit.find('target').renderContents('utf-8').strip()

            # Note: We don't use xliff to remove values, so no value means no
            # translation and no change to the original
            # XXX: This doesn't handle values other than strings, this may be
            # a problem.
            if not value:
                continue

            # convert HTML entities
            value = safe_unicode(value)
            value = html_parser.unescape(value)
            values[fieldname] = value

        if IDexterityContent.providedBy(target_ob):
            # Dexterity
            schema = get_dx_schema(target_ob)
            for name, field in schema.items():

                if ILanguageIndependentField.providedBy(field):
                    # Copy from the original:
                    field.set(target_ob, field.get(source_ob))
                    pass
                elif name in values:
                    value = values[name]
                    if IRichText.providedBy(field):
                        it = getattr(source_ob, name)
                        if it is not None:
                            mimeType = it.mimeType
                            outputMimeType = it.outputMimeType
                        else:
                            # Default
                            mimeType = 'text/html'
                            outputMimeType = 'text/x-html-safe'
                        value = RichTextValue(value, mimeType, outputMimeType)
                    schema[name].set(target_ob, value)

        self.total += 1


@implementer(IXLIFFExporter)
class XLIFFExporter(object):
    """ Adapter to generate an xliff representation from a content object """

    recursive = False
    single_file = True
    html_compatibility = False
    zip = False
    source_language = "en"

    def __init__(self, context):
        self.context = context

    def _getObjectsByPath(self, ob=None):
        catalog = getToolByName(self.context, 'portal_catalog')
        if ob is None:
            ob = aq_inner(self.context)
        if not ob.isPrincipiaFolderish:
            # context is not a folder but we should work recursive
            # => use parent folder
            path = "/".join(ob.getPhysicalPath()[:-1])
        else:
            path = "/".join(ob.getPhysicalPath())

        object_provides = "plone.app.multilingual.interfaces.ITranslatable"
        results = catalog(path=path, object_provides=object_provides)
        # make a real list out the LazyMap
        results = list(results)
        # sort by path, so that top-level object will get translated first

        results.sort(key=lambda x: len(x.getPath().split('/')))

        return [r.getObject() for r in results]

    def export(self):
        if self.recursive is True:
            objects = self._getObjectsByPath()
        else:
            objects = [self.context]

        if self.html_compatibility:
            HEAD = HTML_HEAD
        else:
            HEAD = XLIFF_HEAD

        xliff_pages = []

        for ob in objects:
            # Make extra sure the object is translatable. The multilingual
            # behavior for this type might have been deactivated without reindexing
            # all instances of this type.
            if not ITranslatable.providedBy(ob):
                continue
            xob = IXLIFF(ob)
            xliff_pages.append((
                "/".join(ob.getPhysicalPath()),
                xob.render(self.html_compatibility, self.source_language),))

        if self.zip is True:
            Z = BytesIO()
            zf = zipfile.ZipFile(Z, 'w')

            if self.single_file is True:    # single file as zip
                if len(xliff_pages) == 1:
                    zf.writestr(
                        '{0}.xliff'.format(xliff_pages[0][0]), HEAD.format(content=xliff_pages[0][1]))
                else:
                    data = [x[1] for x in xliff_pages]
                    zf.writestr(
                        'export.xliff', HEAD.format(content="\n".join(data)))

            # multiple files as zip
            else:
                for page in xliff_pages:
                    zf.writestr('{0}.xliff'.format(page[0]), HEAD.format(content=page[1]))

            zf.close()
            Z.seek(0)
            content = Z.read()
            Z.close()

        else:
            # files as plain, so we need to join them to single_file.
            # one or more files to be returned directly, so we don't need
            # the filenames
            content = [x[1] for x in xliff_pages]
            content = "\n".join(content)
            content = HEAD.format(content=content)
            # Set that as it has been selected implicitly by specifying
            # multiple files but not selecting zip
            self.single_file = True

        return content


@adapter(ITranslatable)
@implementer(IXLIFF)
class XLIFF(object):
    """ """

    def __init__(self, context):
        self.context = context

    def render(self, html_compatibility, source_language):
        context = aq_inner(self.context)

        # lookup adapter
        adapter = IAttributeExtractor(context)
        attrs = adapter.get_attrs(html_compatibility, source_language)

        data = dict(original="/".join(context.getPhysicalPath()),
                    oid=context.UID(),
                    source_language=source_language,
                    attrs="\n".join(attrs),)

        if html_compatibility:
            filedata = HTML_FILE_BODY.format(**data)
        else:
            filedata = XLIFF_FILE_BODY.format(**data)

        return filedata


def _guessLanguage(filename):
    """
    try to find a language abbreviation in the string
    acceptable is a two letter language abbreviation at the start of the
    string followed by an _
    or at the end of the string prefixed by an _ just before the extension
    or preceded and followed by an _
    """

    site = getSite()
    portal_languages = getToolByName(site, 'portal_languages')
    langs = portal_languages.getSupportedLanguages()

    if len(filename) > 3 and filename[2] in ['_', '-']:
        lang = filename[0:2].lower()
        if lang in langs:
            return lang
    if len(filename) > 3 and '.' in filename:
        elems = filename.split('.')
        stem = ".".join(elems[:-1])
        if len(stem) > 3 and stem[-3] in ['_', '-']:
            lang = stem[-2:].strip().lower()
            if lang in langs:
                return lang
        elif len(stem) == 2:
            lang = stem.lower()
            if lang in langs:
                return lang
        for elem in stem.split('_')[1:-1]:
            if len(elem) == 2:
                lang = elem.lower()
                if lang in langs:
                    return lang
    return ''



# -*- coding: utf-8 -*-

from Acquisition import aq_inner
from plone.app.contenttypes.interfaces import ICollection
from plone.app.contenttypes.interfaces import IDocument
from plone.app.contenttypes.interfaces import IEvent
from plone.app.contenttypes.interfaces import ILink
from plone.app.contenttypes.interfaces import INewsItem
from plone.app.multilingual.dx.interfaces import ILanguageIndependentField
from plone.app.textfield.interfaces import IRichTextValue
from plone.dexterity.interfaces import IDexterityContent
from slc.xliff.interfaces import IAttributeExtractor
from slc.xliff.templates.html import HTML_ATTR_BODY
from slc.xliff.templates.xliff import XLIFF_ATTR_BODY
from slc.xliff.xliff import get_dx_schema
from slc.xliff.xliff import logger
from zope.component import adapter
from zope.interface import implementer
import six



@implementer(IAttributeExtractor)
@adapter(IDexterityContent)
class BaseDXAttributeExtractor(object):
    """ Adapter to retrieve attributes from a standard Dexterity object.
    """


    # If you are writing your own Extractor, inherit from this one and simply
    # override the attrs attribute
    attrs = ['title', 'description']

    def __init__(self, context):
        self.context = aq_inner(context)

    def get_attrs(self, html_compatibility, source_language):

        schema = get_dx_schema(self.context)
        attrs = []

        for key in self.attrs:
            field = schema[key]
            if ILanguageIndependentField.providedBy(field):
                logger.warn(
                    "Exporting language independent attribute %s, "
                    "this may give unexpected results during import such as all "
                    "language versions have the value of the last language set "
                    "in the attribute!", key)

            value = field.get(self.context)
            if IRichTextValue.providedBy(value):
                if value.raw is None:
                    value = ''
                else:
                    value = value.raw
            if isinstance(value, six.binary_type):
                value = value.decode('UTF-8')

            data = dict(id=key,
                        value=value,
                        source_language=source_language)

            if html_compatibility:
                attrs.append(HTML_ATTR_BODY.format(**data))
            else:
                attrs.append(XLIFF_ATTR_BODY.format(**data))

        return attrs


@implementer(IAttributeExtractor)
@adapter(IDocument)
class DocumentAttributeExtractor(BaseDXAttributeExtractor):
    """ Adapter to retrieve attributes from a standard document based
    object """
    attrs = ['title', 'description', 'text']


@implementer(IAttributeExtractor)
@adapter(ICollection)
class TopicAttributeExtractor(BaseDXAttributeExtractor):
    """ Adapter to retrieve attributes from a standard document based
    object """
    attrs = ['title', 'description', 'text']


@implementer(IAttributeExtractor)
@adapter(IEvent)
class EventAttributeExtractor(BaseDXAttributeExtractor):
    """ Adapter to retrieve attributes from a standard event based
    object """
    attrs = ['title', 'description', 'location', 'text']


@implementer(IAttributeExtractor)
@adapter(INewsItem)
class NewsItemAttributeExtractor(BaseDXAttributeExtractor):
    """ Adapter to retrieve attributes from a standard event based
    object """
    attrs = ['title', 'description', 'image_caption', 'text']


@implementer(IAttributeExtractor)
@adapter(ILink)
class LinkAttributeExtractor(BaseDXAttributeExtractor):
    """ Adapter to retrieve attributes from a standard event based
    object """
    attrs = ['title', 'description', 'remoteUrl']



# -*- coding: utf-8 -*-
"""
This module contains the  slc.xliff package
"""
import os
from setuptools import setup, find_packages


def read(*rnames):
    return open(os.path.join(os.path.dirname(__file__), *rnames)).read()

version = '3.0'

long_description = (
    read('README.rst')
    + '\n' +
    read('CHANGES.rst')
    + '\n' +
    read('CONTRIBUTORS.rst')
    + '\n'
    )

tests_require = ['mock', 'manuel', 'plone.app.testing', 'Pillow', 'plone.testing', 'plone.app.robotframework[debug]', 'plone.app.contenttypes']

setup(name='slc.xliff',
      version=version,
      description="A framework to allow im and export of xliff files for translations",
      long_description=long_description,
      # Get more strings from http://www.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        "Framework :: Plone",
        "Framework :: Plone :: 5.0",
        "Framework :: Plone :: 5.1",
        "Framework :: Plone :: 5.2",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "License :: OSI Approved :: GNU General Public License (GPL)",
        "License :: OSI Approved :: European Union Public Licence 1.1 (EUPL 1.1)",
        ],
      keywords='xliff translation import export',
      author='Syslab.com GmbH',
      author_email='info@syslab.com',
      url='https://github.com/collective/slc.xliff',
      license='GPL + EUPL',
      packages=find_packages('src'),
      package_dir={'': 'src'},
      namespace_packages=['slc', ],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          'z3c.autoinclude',
          'zope.component',
          'plone.app.contenttypes',
          'plone.app.multilingual',
          'plone.api',
          'BeautifulSoup4',
          'six',
          # -*- Extra requirements: -*-
      ],
      tests_require=tests_require,
      extras_require=dict(test=tests_require, plone4=['Zope2']),
      test_suite='slc.xliff.tests.test_docs.test_suite',
      entry_points="""
        [z3c.autoinclude.plugin]
        target = plone
      """,
      )



# -*- coding: utf-8 -*-
"""Setup tests for this package."""
from bs4 import BeautifulSoup
from bs4 import NavigableString
from plone import api
from plone.app.multilingual.interfaces import ITranslationManager
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.textfield.value import RichTextValue
from plone.dexterity.utils import createContentInContainer
from plone.namedfile.file import NamedFile
from six import StringIO
from slc.xliff.interfaces import IXLIFFExporter
from slc.xliff.interfaces import IXLIFFImporter
from slc.xliff.tests.base import FUNCTIONAL_TESTING
from slc.xliff.tests.base import INTEGRATION_TESTING
from zope.component import getUtility
from Products.CMFPlone.utils import safe_unicode
from zipfile import ZipFile
from tempfile import mktemp

import unittest


try:
    from Products.CMFPlone.utils import get_installer
except ImportError:
    get_installer = None


def prepare_contents_in_language(soup, lang="de", lang_name="german"):
    """ helper function to create an xliff file with contents translated in the
    given translation with dummy texts showing the language
    """

    new_title = NavigableString("My {0} Title".format(lang_name))
    soup.find("trans-unit", attrs={"id": "title"}).findNext("target").append(new_title)
    new_description = NavigableString("My {0} Description".format(lang_name))
    soup.find("trans-unit", attrs={"id": "description"}).findNext("target").append(
        new_description
    )
    new_text = NavigableString("<p>My {0} Text</p>".format(lang_name))
    soup.find("trans-unit", attrs={"id": "text"}).findNext("target").append(new_text)
    xliffstr = soup.prettify()
    xliffstr = xliffstr.replace(
        '<target xml:lang="en">', '<target xml:lang="{0}">'.format(lang)
    )
    xliffstr = xliffstr.replace(
        'target-language=""', 'target-language="{0}"'.format(lang)
    )

    return xliffstr


class TestXliffExport(unittest.TestCase):

    layer = INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests."""
        self.portal = self.layer["portal"]
        self.doc_en = createContentInContainer(
            self.portal["en"],
            "test_content",
            checkConstraints=False,
            id="doc_en",
            title="My english title",
            description="My english description",
        )
        self.doc_en.text = RichTextValue("<p>My english text</p>")

    def test_create_xliff_representation_links_to_object(self):
        xliffexporter = IXLIFFExporter(self.doc_en)
        xliffexporter.recursive = False
        xliffexporter.single_file = True
        xliffexporter.html_compatibility = False
        xliffexporter.zip = False
        xliffexporter.source_language = "en"
        data = xliffexporter.export()
        soup = BeautifulSoup(data)

        self.assertEqual(soup.findAll("file")[0]["oid"], self.doc_en.UID())

    def test_all_attributes_are_in_the_xliff_file(self):
        xliffexporter = IXLIFFExporter(self.doc_en)
        xliffexporter.recursive = False
        xliffexporter.single_file = True
        xliffexporter.html_compatibility = False
        xliffexporter.zip = False
        xliffexporter.source_language = "en"
        data = xliffexporter.export()
        soup = BeautifulSoup(data)
        file_attributes = [x["id"] for x in soup.findAll("trans-unit")]
        self.assertIn("title", file_attributes)
        self.assertIn("description", file_attributes)
        self.assertIn("text", file_attributes)


class TestXliffImport(unittest.TestCase):

    layer = FUNCTIONAL_TESTING

    def setUp(self):
        """Custom shared utility setup for tests."""
        self.portal = self.layer["portal"]
        self.doc_en = createContentInContainer(
            self.portal["en"],
            "test_content",
            checkConstraints=False,
            id="doc_en",
            title="My english title",
            description="My english description",
        )

        self.doc_en.text = RichTextValue("<p>My english text</p>")
        self.doc_en.language_independent = "This should not change."

        import transaction

        transaction.commit()

    def test_upload_single_content(self):
        """ We export the XLIFF file, change some contents and upload it to create
        the translated content
        """

        xliffexporter = IXLIFFExporter(self.doc_en)
        xliffexporter.recursive = False
        xliffexporter.single_file = True
        xliffexporter.html_compatibility = False
        xliffexporter.zip = False
        xliffexporter.source_language = self.doc_en.Language()
        xliffstr = xliffexporter.export()
        soup = BeautifulSoup(xliffstr, "xml")

        xliffstr_de = prepare_contents_in_language(soup, "de", "german")

        xliffimporter = getUtility(IXLIFFImporter)
        xliff_file = NamedFile(
            data=xliffstr_de, contentType="text/xml", filename=u"transl_de.xliff"
        )

        xliffimporter.upload(xliff_file, html_compatibility=False)

        doc_de = ITranslationManager(self.doc_en).get_translation("de")

        self.assertIsNotNone(doc_de)
        self.assertEqual(doc_de.Title(), "My german Title")
        self.assertIn("My german Description", doc_de.Description())
        self.assertEqual(doc_de.text.output, "<p>My german Text</p>")
        self.assertEqual(doc_de.language_independent, "This should not change.")

    def test_upload_multiple_contents(self):
        """ create contents in spanish and finish and upload as it was a single file """
        xliffexporter = IXLIFFExporter(self.doc_en)
        xliffexporter.recursive = False
        xliffexporter.single_file = True
        xliffexporter.html_compatibility = False
        xliffexporter.zip = False
        xliffexporter.source_language = self.doc_en.Language()
        xliffstr = xliffexporter.export()

        soup_es = BeautifulSoup(xliffstr, "xml")
        xliffstr_es = prepare_contents_in_language(soup_es, "es", "spanish")

        soup_fi = BeautifulSoup(xliffstr, "xml")
        xliffstr_fi = prepare_contents_in_language(soup_fi, "fi", "finnish")

        xliffstr_translations = xliffstr_es + xliffstr_fi

        xliffimporter = getUtility(IXLIFFImporter)
        xliff_file = NamedFile(
            data=xliffstr_translations,
            contentType="text/xml",
            filename=u"transl_mixed.xliff",
        )

        xliffimporter.upload(xliff_file, html_compatibility=False)

        doc_es = ITranslationManager(self.doc_en).get_translation("es")

        self.assertIsNotNone(doc_es)
        self.assertEqual(doc_es.Title(), "My spanish Title")
        self.assertIn("My spanish Description", doc_es.Description())
        self.assertEqual(doc_es.text.output, "<p>My spanish Text</p>")
        self.assertEqual(doc_es.language_independent, "This should not change.")

        doc_fi = ITranslationManager(self.doc_en).get_translation("fi")

        self.assertIsNotNone(doc_fi)
        self.assertEqual(doc_fi.Title(), "My finnish Title")
        self.assertIn("My finnish Description", doc_fi.Description())
        self.assertEqual(doc_fi.text.output, "<p>My finnish Text</p>")
        self.assertEqual(doc_fi.language_independent, "This should not change.")

    def test_upload_zipfile(self):
        xliffexporter = IXLIFFExporter(self.doc_en)
        xliffexporter.recursive = False
        xliffexporter.single_file = True
        xliffexporter.html_compatibility = False
        xliffexporter.zip = False
        xliffexporter.source_language = self.doc_en.Language()
        xliffstr = xliffexporter.export()

        soup_es = BeautifulSoup(xliffstr, "xml")
        xliffstr_es = prepare_contents_in_language(soup_es, "es", u"spanish zipped")

        soup_fi = BeautifulSoup(xliffstr, "xml")
        xliffstr_fi = prepare_contents_in_language(soup_fi, "fi", u"finnish zipped")

        tempfilename = mktemp(suffix=".zip")
        with open(tempfilename, "wb") as out:
            zf = ZipFile(out, "w")
            zf.writestr("spanish.xliff", xliffstr_es)
            zf.writestr("finnish.xliff", xliffstr_fi)
            zf.close()

        with open(tempfilename, "rb") as out:
            xliffimporter = getUtility(IXLIFFImporter)
            zip_file = NamedFile(
                data=out, contentType="application/zip", filename=u"transl_mixed.zip"
            )
            xliffimporter.upload(zip_file, html_compatibility=False)
            doc_es = ITranslationManager(self.doc_en).get_translation("es")
            self.assertEqual(doc_es.Title(), "My spanish zipped Title")

            doc_fi = ITranslationManager(self.doc_en).get_translation("fi")
            self.assertEqual(doc_fi.Title(), "My finnish zipped Title")
